package Sophos::Reports::Report::PMX;
use strict;
use warnings;
use base 'Sophos::Reports::Report';
use PureMessage::Config qw/%config/;
use PureMessage::Database::Config;
use PureMessage::WebUI::I18N;
use Carp qw(croak);
use DBI;

sub new {
    my ($class, %opts) = @_;
    my $self = $class->SUPER::new(%opts);

    # Get language-specific nodata image directory
    my $lh = PureMessage::WebUI::I18N->get_handle_via_config();
    my %image_info = $lh->image_info($self->{nodata} . '.png');
    $self->{nodata_dir} = $image_info{static_dir};
    
    $self->{span} ||= 'today';

    unless ($self->{date}) {
        if ($self->{span} =~ /hour/) {
            my $hour_from_now = time + 3600;
            # Ensure that hour-based reports have nice buckets
            my ($year, $month, $day, $hour) = (localtime($hour_from_now))[5,4,3,2];
            $self->{date} = sprintf '%04d-%02d-%02dT%02d:00',
                                    $year + 1900, $month + 1, $day, $hour;
        }
        else {
            # spans bigger than hours should be based on tomorrow
            $self->{date} = 'Tomorrow';
        }
    }

    my $groups = ref($opts{groups}) eq 'ARRAY' ? $opts{groups} : [$opts{groups}];
    $self->{groups} = defined $opts{groups} ? $groups : [];
    $self->{only_group} = $opts{only_group};
    $self->{uri_prefix} = '/groups/report-img';
    $self->{hide_groups} = 0;

    $self->{lang} ||= 'en';
    ($self->{report} = $class) =~ s/.+:://;

    if (my $dir = $self->{direction}) {
        die "'$dir' is not a valid direction" unless $dir =~ /^(?:in|out)bound$/;
    }

    return $self;
}

sub hide_groups { 
    my $self = shift;
    return $self->{hide_groups} || 0;
}

sub set_groups {
    my ($self, $groups) = @_;
    $self->{groups} = $groups;
}

sub _group_sql {
    my ($self) = @_;

    my $group_sql = '';

    if (defined($self->{only_group})) {
        $group_sql = qq{AND group_name = '$self->{only_group}'};
    }
    elsif ($self->{groups} and @{$self->{groups}}) {
        $group_sql = "AND ("
            . join(' OR ', map { "group_name = '$_'" } @{$self->{groups}})
            . ")";
    }
    else {
        $group_sql = "AND (group_name = '__NO_GROUP__')";
    }
    return $group_sql;
}

sub dbh {
    my $self = shift;
    return $self->{dbh} if defined $self->{dbh} && $self->{dbh}->ping();

    my $db_config = PureMessage::Database::Config->new();
    my $stores = $db_config->stores();
    my $reports_data_store = $stores->{'pmx_prd_store'} || 'pmx_quarantine';
    my $dsn = $db_config->store_value($reports_data_store, 'dsn');

    # NOTE: We ensure that all database handles open using exactly the same
    # connection arguments so that DBI handles can be shared among multiple
    # packages.  See the following docs for more details.
    # http://perl.apache.org/docs/1.0/guide/databases.html#Opening_Connections_With_Different_Parameters
    my $connect_method = $ENV{MOD_PERL} ? "connect" : "connect_cached";
    $self->{dbh} = DBI->$connect_method($dsn, "", "", { RaiseError  => 0,
                                                        PrintError  => 0,
                                                        AutoCommit  => 0, 
                                                      }
                                        );
    croak("Unable to connect to DB using dsn '$dsn'") unless $self->{dbh};

    # Override the default ping method, always talk to the server.
    # Dont rely on transaction state alone.
    $self->{dbh}->{Callbacks} = {
        ping => sub {
             undef $_;

             my ($dbh) = @_;
             my $ret = 0;
             eval {
                 local $SIG{__DIE__}  = sub { return 0 };
                 local $SIG{__WARN__} = sub { return 0 };
                 $ret = $dbh->do("select 'DBD::Pg::ping test'");
             };
             return $@ ? 0 : $ret;
        },
    };

    $self->{dbh}{RaiseError}  = 1; 
    $self->{dbh}{PrintError}  = 0; 
    $self->{dbh}{AutoCommit}  = 0; 
    $self->{dbh}{HandleError} = undef;
    return $self->{dbh};
}

sub format { $_[0]->{type} }

sub legend_data {
    my ($self) = @_;
    my $leg = $self->{legend} || die "No legend specified!";

    my @legend;
    for my $i (0 .. $#$leg) {
        push @legend, {
            label => $leg->[$i],
            color => $self->{graph}{dclrs}[$i]
        };
    }
    return \@legend;
}

sub DESTROY {
    my $self = shift;
    $self->{dbh}->disconnect if $self->{dbh};
}

sub views {
    return $_[0]->{views}
}

sub has_direction { 1 }

sub direction { $_[0]->{direction} || '' }
sub history   { $_[0]->{history}   || '' }
sub date      { $_[0]->{date}      || '' }
sub span      { $_[0]->{span}      || '' }
sub groups    { $_[0]->{groups} ? join(',', @{$_[0]->{groups}}) : '' }
sub only_group { $_[0]->{only_group} || '' }
sub start_date { $_[0]->{date_range_start} }
sub end_date   { $_[0]->{date_range_end} }

sub img_src {
    my $self = shift;
    return undef unless $self->{views}{bars} || $self->{views}{lines};
    (my $class = ref($self)) =~ s#.+::(.+)#$1#;
    (my $span = $self->{span}) =~ s#\s##g;
    $span .= ' sfn=1' if $self->{sfn};
    (my $date = $self->{date}) =~ s#\s#T#;
    my $url = "$class/$self->{type}/$date/$span.png";
    my $args = $self->_extra_url_args;

    return "$self->{uri_prefix}/$url$args";
}

sub export_src {
    my $self = shift;
    (my $class = ref($self)) =~ s#.+::(.+)#$1#;
    (my $span = $self->{span}) =~ s#\s##g;
    $span .= ' sfn=1' if $self->{sfn};
    (my $date = $self->{date}) =~ s#\s#T#;
    my $url = "$class/$self->{type}/$date/$span.csv";
    my $args = $self->_extra_url_args;
    return "$self->{uri_prefix}/$url$args";
}

sub _extra_url_args {
    my $self = shift;
    my @args;
    if ($self->{only_group}) {
        push @args, "only_group=$self->{only_group}";
    }
    if ($self->{direction}) {
        push @args, "direction=$self->{direction}";
    }
    if (exists $self->{mkey} && defined $self->{mkey}) {
        push @args, "mkey=$self->{mkey}";
    }

    return '?' . join('&', @args) if @args;
    return '';
}

sub table_data {
    my $self = shift;
    my $data = $self->data(no_flop => 1);
    # create row subtotals and overall totals
    my @totals;
    for my $d (@$data) {
        my $subtotal = 0;
        for my $col (1 .. $#$d) {
            my $value = $d->[$col];
            $totals[$col] += $value;
            $subtotal += $value;
        }
        push @$d, $subtotal if $self->{table_row_subtotals};
    }
    shift @totals;

    if ($self->{table_row_subtotals}) {
        # add a totals total
        my $total_total = 0;
        $total_total += $_ for @totals;
        push @totals, $total_total;
    }

    $self->{table_totals} = \@totals;
    return $data;
}

sub table_totals {
    my $self = shift;
    return $self->{table_totals}
                || die "No totals calculated! Call table_data first!";
}

sub log {
    my ($self, $msg) = @_;
    if ($self->{logger}) {
        $self->{logger}->($msg);
    }
    else {
        warn "$msg\n";
    }
}

1;
